# Copyright(c) 2002-2009 Association of Universities for Research in Astronomy, Inc.
#
# GMOS example reductions script: Typical reduction of imaging data
#
# This data processing was done to aid the data quality assessment performed
# by the Gemini staff. The data processing is not designed to give the best
# possible result. Better signal-to-noise and better cleaning for cosmic-ray
# hits and bad pixels will most likely be possible. The user of these data
# is encouraged to use the provided co-added images only as guide lines and
# to re-reduce the data to obtain the best possible reduction.
#
# Gemini GMOS data reduction script
# Observation UT date:  2002jun06
# Data processor:  Inger Jorgensen
# Data reduction date:  2002jun07
# Revisions: 2002aug26 garith changed to gemarith
#            2004oct11 changed to use gifringe and girmfringe
#
# Brief data description: Imaging for the SV program GN-2002A-SV-78

gmos.logfile="20020606_GN-2002A-SV-78.log"

# Locaion of calibration files
set calib=/net/tyl/export/data2/gmos/2002jun14/

delete glist ver-
for(i=140; i<=142; i+=1) 
 print("N20020606S0000"+i, >> "glist")

#bias and flat have been overscan corrected

gireduce @glist bias=calib$Basecalib/N20020606S0173_bias \
flat1=calib$Basecalib/N20020606S0149_flat \
rawpath="gdata$2002jun06/" fl_over+

delete rglist ver-
sections rg//@glist > rglist

# Make a fringe frame
fields rglist 1 lines="1" | scan(s1)
s1=s1//"_frg"
gifringe("@rglist",s1)

# Subtract off the fringe frame
girmfringe("@rglist",s1,scale=1.0)
delete frglist ver-
sections f//@rglist > frglist

# Mosaic the frames
gmosaic @frglist 

# Coadd the frames, the default bpm is found automatically
delete mrglist ver-
sections m//@frglist > mrglist
imcoadd @mrglist fwhm=9 threshold=100 \
logfile="20020606_GN-2002A-SV-78.log" 

gemseeing mfrgN20020606S0140_add fl_inter+ \
logfile=20020606_GN-2002A-SV-78.log fl_over+
gemqa  mfrgN20020606S0140_add pass pass pass pass \
logfile=20020606_GN-2002A-SV-78.log

