# Copyright(c) 2002-2012 Association of Universities for Research in Astronomy, Inc.

procedure gdisplay (image, frame)

# Display GMOS (raw) images
#
# Version Feb 28, 2002   IJ  v1.3 release
#         Apr 26, 2002   IJ  change warning comment, handle [1] images
#         Jul  7, 2002   IJ  handle dir-names+short image names correctly
#         Sept 20, 2002  IJ   v1.4 release
#         Oct 17, 2002   BM  generalized for GMOS-S, 6amps, and all readmodes
#         Oct 22, 2002   BM  fix problem with defining l_gainmode when fl_bias-
#         Jan 31, 2003   BM  Added output file
#         Sep 13, 2006   JH  Add image number feature as input image if current UT
#         Aug 02, 2009   AS  Set saturation limits based on binning
#         Sep 22, 2009   AS  Set saturation limits based on twilight

char    image       {prompt="GMOS image to display, can use number if current UT"} # OLDP-1-input-primary-none
int     frame       {1,prompt="Frame to write to"}                                 # OLDP-1
char    output      {"",prompt="Save pasted file to this name if not blank"}       # OLDP-3
bool    fl_paste    {no,prompt="Paste images to one for imexamine"}                # OLDP-2
bool    fl_bias     {no,prompt="Rough bias subtraction"}                           # OLDP-2
char    rawpath     {"adata$",prompt="Path for input image if not included in name"}
char    gap         {"default",prompt="Size of the gap between the CCDs (in pixels)"}     # OLDP-3
real    z1          {0.,prompt="Lower limit if not autoscaling"}                   # OLDP-2
real    z2          {0.,prompt="Upper limit if not autoscaling"}                   # OLDP-2
bool    fl_sat      {no,prompt="Flag saturated pixels"}                            # OLDP-2
bool    fl_imexam   {yes,prompt="If possible, run imexam"}
real    signal      {INDEF,prompt="Flag pixels with signal above this limit"}      # OLDP-2
char    sci_ext     {"all",prompt="Name of extension(s) to display"}               # OLDP-3
char    observatory {")_.observatory", prompt="Observatory (gemini-north or gemini-south)"}
char    prefix      {"auto",prompt="File prefix, (N/S)YYYYMMDDS if not auto"}
char    key_detsec  {"DETSEC",prompt="Header keyword for detector section"}        # OLDP-3
char    key_datasec {"DATASEC",prompt="Header keyword for data section"}           # OLDP-3
char    key_ccdsum  {"CCDSUM",prompt="Header keyword for CCD binning"}             # OLDP-3
char    gaindb      {"default",prompt="Database with gain data"}    # OLDP-2
bool    verbose     {no,prompt="Verbose"}                                          # OLDP-4
int     status      {0,prompt="Exit status (0=good)"}
struct  *scanfile   {"",prompt="Internal use only."}                               # OLDP-4

begin

    char    l_image = ""
    char    l_output = ""
    char    l_rawpath = ""
    char    l_sci_ext = ""
    char    l_observatory = ""
    char    l_prefix = ""
    char    l_key_detsec = ""
    char    l_key_datasec = ""
    char    l_key_ccdsum = ""
    char    l_gaindb = ""
    bool    l_verbose, l_fl_paste, l_fl_bias, l_fl_sat, l_fl_imexam
    int     l_frame
    real    l_z1, l_z2, l_signal

    char    tmpin, l_dsec, l_datasec, l_stdimage, junk, tmpout, l_paste
    char    tmpim, l_imext, tmpdisp, keyfound, gaindbname
    char    tmpsignal, tmpbpm, tmpxbpm, tmpxsig
    char    l_pixtype, l_gap
    int     n_ext, Xmax, Ymax, x1, x2, y1, y2, n_i, n_j, n_k
    int     Xoff, Yoff, Xbin, Ybin, gapvalue, Xlen
    real    l_xcenter, l_ycenter, l_mag, imX, imY
    bool    fl_erase, fl_zscale, fl_zrange, l_fl_mag, l_fl_chmag, l_fl_sec
    bool    ishamamatsu, ise2vDD, canbias
    real    l_bias[100], bias, l_gain, l_rdnoise, l_maxbias, l_saturation
    string  l_readmode, l_gainmode, dum, s_bias, dbpm, doverlay, l_express, amp
    string  object, grating
    int     l_igain, xcenval
    char    lastchar, siteprefix, utdate, imstring, cursorinput, fileprefix
    int     newyoffset, Ymaxtrim, row_removal_value, det_roi_y_start
    int     l_gapvalue
    real    gainmode_check, l_stand_sat, l_reduced_sat, l_print_saturation
    char    readout_mode_check, units, l_sciext, bias_string
    int     counter
    char    e2v_ampnames[6] = "e2v 10031-23-05, left","e2v 10031-23-05, right","e2v 10031-01-03, left","e2v 10031-01-03, right","e2v 10031-18-04, left","e2v 10031-18-04, right"
    real    e2v_FWS[6] = 47500.0,47000.0,54000.0,50000.0,52000.0,47500.0 # full--well saturation slow/loe mode 1x1 binning

    junk = fscan (image,       l_image)
    junk = fscan (output,      l_output)
    junk = fscan (rawpath,     l_rawpath)
    junk = fscan (observatory, l_observatory)
    junk = fscan (prefix,      l_prefix)
    junk = fscan (key_detsec,  l_key_detsec)
    junk = fscan (key_datasec, l_key_datasec)
    junk = fscan (key_ccdsum,  l_key_ccdsum)
    junk = fscan (gaindb,      l_gaindb)
    junk = fscan (sci_ext,     l_sci_ext)
    junk = fscan (gap,         l_gap)

    print (l_sci_ext) | translit ("STDIN", "a-z", "A-Z", delete-, collapse-) |\
        scan (l_sci_ext)

    status = 0
    l_z1 = z1
    l_z2 = z2
    l_verbose = verbose
    l_frame = frame
    l_fl_paste = fl_paste
    l_fl_bias = fl_bias
    l_fl_sat = fl_sat
    l_fl_imexam = fl_imexam
    l_signal = signal

    tmpin = mktemp ("tmpin")
    tmpout = mktemp ("tmpout")
    tmpdisp = mktemp ("tmpdisp")
    tmpim = mktemp ("tmpim1")
    tmpbpm = mktemp ("tmpbpm")
    tmpxbpm = mktemp ("tmpxbpm")
    tmpsignal = mktemp ("tmpsignal")
    tmpxsig = mktemp ("tmpxsig")

    # For new GMOS-N Hamamatsu CCDs: number of unused rows at the bottom of the
    # detector
    row_removal_value = 48
    ishamamatsu = no
    ise2vDD = no

    fl_zscale = yes
    fl_zrange = yes
    if ((l_z1 != 0.) || (l_z2 != 0.)) {
        fl_zscale = no
        fl_zrange = no
    }

    cache ("imgets", "gimverify", "fparse", "gemisnumber", "getfakeUT")

    if (l_verbose)
        print ("...checking that the paths have a trailing slash...")
    lastchar = substr(l_rawpath, strlen(l_rawpath), strlen(l_rawpath))
    if ((l_rawpath != "") && (lastchar != "/") && (lastchar != "$"))
        l_rawpath = l_rawpath//"/"

    #-----------------------------------------------------------------------
    # Build file prefix in 'auto' mode

    if (l_prefix == "auto") {

        #### Convert observatory value to lower case
        l_observatory = strlwr (l_observatory)

        #### Determine site prefix
        if (l_observatory == "gemini-north") {
            siteprefix = "N"
        } else if (l_observatory == "gemini-south") {
            siteprefix = "S"
        } else {
            print ("GDISPLAY - ERROR: Parameter 'observatory' must be either \
                'gemini-north' or 'gemini-south'")
            status = 121
            goto crash
        }

        #### Build and set file prefix
        getfakeUT()
        utdate = getfakeUT.fakeUT

        fileprefix = siteprefix//utdate//"S"
    } else {
        fileprefix = l_prefix
    }


#-----------------------------------------------------------------------
# Construct image name

    gemisnumber (l_image, "integer", verbose-)
    if (gemisnumber.fl_istype) {
        print ("GDISPLAY: Constructing image name based on today's UT \
                date or given file prefix.")

        if (l_verbose)
            print ("...prefix = ",fileprefix)
        print ("l_image = ", l_image)

        printf ("%04d\n", int(l_image)) | scan(imstring)
        l_image = fileprefix//imstring//".fits"
        if (l_verbose)
            print ("...image = ",l_image)
    }

    # Is there a directory name
    gimverify (l_image)
    fparse (l_image)
    if (fparse.directory == "") {
        gimverify (l_image) # current directory
        if (gimverify.status != 0)
            gimverify (l_rawpath//l_image) # try rawpath
    }
    if (gimverify.status == 1) {
        print ("ERROR: Cannot access image "//l_image)
        goto crash
    } else if (gimverify.status != 0) {
        print ("ERROR: Image "//l_image//" is not a MEF file")
        goto crash
    }
    l_image = gimverify.outname # make sure .fits is gone

    print ("GDISPLAY: Using image "//l_image)

    # Check that the image is actually a GMOS image
    keypar (l_image//"[0]", "INSTRUME", silent+)
    if (substr(keypar.value,1,4) != "GMOS") {
        print ("ERROR - GDISPLAY: "//l_image//" is not a GMOS image.")
        goto crash
    }

    # Set default values used for Xoff Yoff, l_<x/y>center calculations
    # and gapvalue, gainmode_check and readout_mode_check
    imgets (l_image//"[0]", "DETTYPE", >& "dev$null")
    if (imgets.value == "S10892-01") { # Hamamatsu CCDs
        ishamamatsu = yes
        l_gapvalue = 37 ##M
        readout_mode_check = "2000" ##M
        gainmode_check = 3. ##M
        l_stand_sat = 58000 ##M
        l_reduced_sat = 45000 ##M
        xcenval = 2048 #hcode
        Xlen = 6144 #hcode
        imX = 6400. #hcode
        imY = 4644. #hcode
        Ymax = 4224 #hcode # Changed to reflect new CCD size
        Ymaxtrim = 4176 #hcode # Set for trimmed Hamamatsu data
    } else if (imgets.value == "SDSU II CCD") { # Current EEV2 CCDs
        #Default values for GMOS S and old GMOS N
        l_gapvalue = 37
        readout_mode_check = "1000"
        gainmode_check = 3.
        l_stand_sat = 58000 # Standard saturation limit
        l_reduced_sat = 45000 # Twilight reduced saturation limit
        xcenval = 2048
        Xlen = 6144
        imX = 6400.
        imY = 4644.
        Ymax = 4608
    } else if (imgets.value == "SDSU II e2v DD CCD42-90") { \
        # New e2vDD CCDs
        ise2vDD = yes
        l_gapvalue = 37 ##M
        readout_mode_check = "1000" ##M
        gainmode_check = 3. ##M
        l_reduced_sat = 45000 ##M
        l_stand_sat = 65535 # Controller digitization limit
        xcenval = 2048
        Xlen = 6144
        imX = 6400.
        imY = 4644.
        Ymax = 4608
    } else {
        print ("ERROR - GDISPLAY: Unknown DETTYPE for image "//l_image)
        goto crash
    }

    # Set gapvalue
    if (l_gap == "default") {
        gapvalue = l_gapvalue
    } else
        gapvalue = int(l_gap)

    # testprinting
    # print ("*** Xlen="//Xlen//" imX="//imX//" imY="//imY//" Ymax="//Ymax//" \
    # gap="//gapvalue)
    # print ("xcenval ="//xcenval)

    fparse (gimverify.outname)
    l_imext = fparse.root
    tmpout = tmpout//l_imext
    fparse (l_image)
    if (fparse.extension != ".fits")
        l_image = l_image//".fits"
    if (fl_paste)
        print ("GDISPLAY: Temporary image name "//tmpout)


    # Check if the image is an ROI and if fl_paste+; cannot currently do this
    # if requested
    keypar (l_image//"[0]", "DETRO2X", silent+)
    if (keypar.found && fl_paste) {
        print ("ERROR - GDISPLAY: "//l_image//" contains multiple ROIs and \
            fl_paste=yes.\n"//\
            "                  Cannot display multiple ROIs with \
            fl_paste+.\n"//\
            "                  Set fl_paste=no to display this image.")
        goto crash
    }

    show stdimage | scan (l_stdimage)
    printf ("stdimage = %s\n", l_stdimage)

    if ((substr(l_stdimage,1,7) == "imtgmos") && \
        (l_stdimage != "imtgmosccd")) {
        Xmax = Xlen + (2. * gapvalue)
        Xoff = nint(imX - Xmax) / 2.
        Yoff = nint(imY - Ymax) / 2.
        Xmax = imX
        Ymax = imY
    } else {
        print ("ERROR - GDISPLAY: Please set stdimage to a full display GMOS \
            device")
        print ("                  imtgmos, imtgmos2, imtgmos4 or imtgmos8")
        goto crash
    }
    l_mag = 1.
    if (strlen(l_stdimage) == 8)
        l_mag = 1./real(substr(l_stdimage,8,8))

    if (l_fl_bias || (l_signal != INDEF) || l_fl_sat) {
        # Select and check gaindb file
        if (l_gaindb == "default") {
            ggdbhelper (l_image//"[0]", logfile="dev$null")
            if (ggdbhelper.status != 0)
                goto crash
            gaindbname= osfn(ggdbhelper.gaindb)
        } else {
            gaindbname = osfn(l_gaindb)
        }
        if (access(gaindbname) == no) {
            printlog ("ERROR - GGAIN: Gain database not found", \
                logfile="dev$null", verbose=yes)
            goto crash
        }
    }

    # Check if the image has been cut.  Cannot bias if image has been cut.
    if (l_fl_bias) {
        keyfound = ""
        hselect (l_image//"[0]", "GSCUT", yes) | scan (keyfound)
        if (keyfound != "") { #keyword was found
            print ("WARNING - GDISPLAY: Cannot bias subtract since image has \
                already been 'GSCUT'")
            print ("WARNING - GDISPLAY: Resetting fl_bias to 'no'")
            l_fl_bias = no
        }
    }

    canbias = l_fl_bias
    if (!l_fl_paste && l_fl_bias) {
        print ("WARNING - GDISPLAY: Cannot bias subtract if not pasting")
        canbias = no
    }

    if (l_sci_ext != "ALL") {
        fxhead (l_image) | match ("IMAGE", "STDIN", stop-) | \
            translit ("STDIN", "a-z", "A-Z", delete-, collapse-) | \
            match (l_sci_ext, "STDIN", stop-, > tmpin)
    } else {
        n_k = 0
        fxhead (l_image) | match("IMAGE", "STDIN", stop-) | \
            translit ("STDIN", "a-z", "A-Z", delete-, collapse-) | \
            match ("SCI", "STDIN", stop-) | scan (n_k)
        if (n_k > 0) {
            fxhead (l_image) | match ("IMAGE", "STDIN", stop-) | \
                translit ("STDIN", "a-z", "A-Z", delete-, collapse-) | \
                match ("SCI", "STDIN", stop-, > tmpin)
            print ("Displaying the SCI extensions")
            l_sci_ext = "SCI"
        } else {
            fxhead (l_image) | match ("IMAGE", "STDIN", stop-, > tmpin)
            # raw data
            l_sci_ext = ""
        }
    }

    count (tmpin) | scan (n_ext) # number of extensions

    if (n_ext == 1 && l_fl_paste) {
        print ("WARNING - GDISPLAY: only one science extension found, \
        turning paste off")
        l_fl_paste = no
    }
    # Initialize
    l_fl_chmag = yes  # check magnification on first round
    l_fl_mag = no     # initialize check for changed magnification
    for (n_k = 1; n_k <= n_ext; n_k = n_k+1) {
        l_bias[n_k] = 0.0
    }

    s_bias = ""
    n_k = 0
    l_fl_sec = yes
    scanfile = tmpin
    while (fscan(scanfile, n_i) != EOF) {
        n_k = n_k + 1

        # Get binning
        imgets (l_image//"["//str(n_i)//"]", l_key_ccdsum)
        if (imgets.value == "0") {
            Xbin = 1
            Ybin = 1
        } else {
            print (imgets.value) | \
                fields ("STDIN", "1,2", lines="1", quit_if_miss-,
                print_file-) | scan (Xbin, Ybin)
        }

        l_dsec = "0"
        l_datasec = "0"
        hselect (l_image//"["//str(n_i)//"]", l_key_detsec, yes) | \
            scan (l_dsec)
        hselect (l_image//"["//str(n_i)//"]", l_key_datasec, yes) | \
            scan (l_datasec)
        if ((l_dsec == "0") || (l_datasec == "0")) {
            l_fl_sec = no
            l_dsec = ""
            l_datasec = ""
        }

        # Cannot show saturated pixels if image is already mosaiced - MS
        ##M Possibly use DQ plane if it contains saturated pixels (value==4)
        keypar (l_image//"[0]", "GMOSAIC", silent+)
        if (keypar.found && l_fl_sat && ise2vDD) {
            print ("WARNING - GDISPLAY: Image has been gmosaic'ed and fl_sat \
                is yes.\n                    Switching fl_sat to no.")
            l_fl_sat = no
        }

        # Get the readout mode (slow/fast lowgain modes only)
        # also needed if flagging based on signal
        if (l_fl_bias || (l_signal != INDEF) || l_fl_sat) {
            if (n_k == 1) {
                imgets (l_image//"[0]", "AMPINTEG", >& "dev$null")
                if (imgets.value == readout_mode_check)
                    l_readmode = "fast"
                else
                    l_readmode = "slow"

                # Set gainmode (high or low)
                # If GAINORIG is there use it, otherwise use GAIN
                keypar (l_image//"["//str(n_i)//"]", "GAINORIG", silent+)
                if (!keypar.found)
                    imgets (l_image//"["//str(n_i)//"]", "GAIN", >& "dev$null")
                else
                    imgets (l_image//"["//str(n_i)//"]", "GAINORIG",
                        >& "dev$null")

                if (real(imgets.value) > gainmode_check)
                    l_gainmode = "high"
                else
                    l_gainmode="low"

            }

            imgets (l_image//"["//n_i//"]", "AMPNAME", >& "dev$null" )
            l_gain = 0.0
            l_rdnoise = 0.0
            match (l_readmode, gaindbname, stop-) | \
                match (l_gainmode, "STDIN", stop-) | \
                match (imgets.value, "STDIN", stop-) | \
                scan (dum, dum, l_gain, l_rdnoise, bias, amp)
            l_bias[n_k] = bias
            if (n_k > 1)
                l_maxbias = max (l_bias[n_k], l_bias[n_k-1])
            else
                l_maxbias = l_bias[n_k]
        }

        if ((l_fl_paste || l_fl_bias) && !l_fl_sec) {
            print ("WARNING - GDISPLAY: No data section info found or image \
                is already mosaiced.")
            print ("                    Displaying input image. Using \
                fl_paste=no fl_bias=no")
            l_fl_paste = no
            l_fl_bias = no
        }

        # Make overlay images as needed
        if (l_fl_sat) {

            if (n_k == 1) {
                tmpxbpm = mktemp ("tmpxbpm")
                copy (l_image, tmpxbpm//".fits", verbose-)

                hselect (l_image//"[0]", "OBJECT", yes) | scan (object)
                if (l_verbose)
                    print ("...object = ", object)
                object = strupr(object)
                hselect (l_image//"[0]", "GRATING", yes) | scan (grating)
                if (l_verbose)
                    print ("...grating = ", grating)

                if ( strstr("TWILIGHT", object) > 0 && grating == "MIRROR" && \
                    Xbin == 1 && Ybin == 1 && !ise2vDD) {
                    print ("GDISPLAY: Imaging twilight flat binned 1x1, \
                        reducing saturation level")
                    l_saturation = l_reduced_sat
                } else {
                    l_saturation = l_stand_sat
                }
                if (l_verbose && !ise2vDD)
                    print ("...saturation level = ", l_saturation)
            }

            # the next bit is a hack for now, it will be done more elegantly
            # in the near future - MS
            if (ise2vDD) {
                if (n_k == 1) {
                    if (l_verbose) {
                        print ("GDISPLAY: e2vDD CCDs detected...")
                        print ("          setting saturation according to "//\
                            "amplifier")
                    }
                    if (!fl_paste) {
                        print ("GDISPLAY: Saturated pixels flagged as red")
                    }
                }

                l_saturation = 0.0
                # Set the saturation according to amplifier
                keypar (l_image//"["//n_i//"]", "AMPNAME", silent+)
                for (counter=1; counter <= 6; counter += 1) {
                    if (keypar.value == e2v_ampnames[counter]) {
                        l_saturation = e2v_FWS[counter]
                        break
                    }
                }

                keypar (l_image//"[0]", "GPREPARE", silent+)
                if (keypar.found) {
                    l_sciext = l_sci_ext//","
                } else {
                    l_sciext = ""
                }

                keypar (l_image//"["//l_sciext//n_i//"]", "GAIN", silent-)
                if (real(keypar.value) == 1.) {
                    # Image is in electrons
                    # Can't deal with this at the moment don't have gain values
                    units = "electrons"
                } else {
                    units = "ADU"
                }
                # Adjust saturation value for binning
                l_saturation = real(Xbin*Ybin*l_saturation)

                # Remember image hasn't been bias subtracted by this point
                # imexpression saturation value to be tested against needs
                # the bias values included
                l_saturation += l_bias[n_i]

                # Now test if saturation is limited by the controller
                l_saturation = min (l_stand_sat, l_saturation)

                # Set the saturation value that will be printed to screen
                l_print_saturation = l_saturation

                # Adjust the value for the bias
                keypar (l_image//"["//l_sciext//n_i//"]", "OVERSCAN", silent+)
                if (!keypar.found) {
                    # Not overscan subtracted
                    if (!canbias) {
                        # Cannot bias subtract
                        bias_string = " (inc. bias)"
                    } else {
                        # Can bias subtract
                        bias_string = " (not inc. bias)"
                        l_print_saturation -= l_bias[n_i]
                    }
                } else {
                    # Overscan subtracted
                    bias_string = " (not inc. bias)"
                    ## Will have to make this weary of units in the future
                    if (l_saturation == l_stand_sat) {
                        # Remove the overscan level
                        l_saturation -= real(keypar.value)
                    } else {
                        # Remove already added bias level
                        l_saturation -= l_bias[n_i]
                    }

                    l_print_saturation = l_saturation

                    if (canbias && n_k == 1) {
                        if (l_verbose) {
                            print ("          Image has already been bias \
                                subtracted setting fl_bias=no")
                        }
                        l_fl_bias = no
                    } else {
                        if (l_verbose) {
                            print ("          Image has previously been bias \
                                subtracted")
                        }
                    }
                }

                if (l_verbose) {
                    print ("... saturation for amplifier: \""//\
                        e2v_ampnames[counter]//"\": is: "//\
                        str(l_print_saturation)//\
                        " ["//units//"]"//bias_string)
                }
            }

            # Image hasn't been overscan subtracted by this point!
            l_express = "a>="//str(l_saturation)//" ? 1 : 0"
            imexpr (l_express, tmpxbpm//"["//str(n_i)//",overwrite]",
                l_image//"["//str(n_i)//"]", verbose-)
            gemhedit (tmpxbpm//"["//str(n_i)//"]", "EXTNAME", "DQ", "", \
                delete-)
            gemhedit (tmpxbpm//"["//str(n_i)//"]", "EXTVER", str(n_k), "",
                delete-)
        }
        if (l_signal != INDEF) {
            if (n_k == 1) {
                tmpxsig = mktemp ("tmpxsig")
                copy (l_image, tmpxsig//".fits", verbose-)
            }
            l_express = "a>="//str(l_signal+l_bias[n_k])//" ? 1 : 0"
            imexpr (l_express, tmpxsig//"["//str(n_i)//",overwrite]",
                l_image//"["//str(n_i)//"]", verbose-)
            gemhedit (tmpxsig//"["//str(n_i)//"]", "EXTNAME", "VAR", "",
                delete-)
            gemhedit (tmpxsig//"["//str(n_i)//"]", "EXTVER", str(n_k), "",
                delete-)
        }
        dbpm = ""
        doverlay = ""

        if (!l_fl_paste) {
            if (l_fl_sec) {
                # find x1, x2, y1, y2 and set display parameters
                n_j = stridx (":", l_dsec)
                x1 = int(substr (l_dsec, 2, n_j-1))
                l_dsec = substr (l_dsec, n_j+1, strlen(l_dsec))
                n_j = stridx (",", l_dsec)
                x2 = int(substr (l_dsec, 1, n_j-1))
                l_dsec = substr (l_dsec, n_j+1, strlen(l_dsec))
                n_j = stridx (":", l_dsec)
                y1 = int(substr (l_dsec, 1, n_j-1))
                l_dsec = substr (l_dsec, n_j+1, strlen(l_dsec))
                y2 = int(substr (l_dsec, 1, strlen(l_dsec)-1))
            } else {
                x1 = 1
                y1 = 1
                hselect (l_image//"["//str(n_i)//"]", "i_naxis1,i_naxis2",
                    yes) | scan (x2, y2)
                x2 = x2 * Xbin
                y2 = y2 * Ybin
            }

            if (n_k == 1)
                fl_erase = yes
            else
                fl_erase = no

            l_xcenter = (x2-x1)/2. + x1
            if (l_xcenter > xcenval)
                l_xcenter += gapvalue
            if ( l_xcenter > ( 2 * xcenval + gapvalue ) )
                l_xcenter += gapvalue
            l_xcenter += Xoff

            l_xcenter = l_xcenter / Xmax

            # If displaying new GMOS-N Hamamatsu CCD data, need to reset
            # Yoff and calculate the variable newyoffset depending on
            # if the data has had the lower 48 rows (unbinned) removed; for
            # ROI stamp images, ROI spectra and images who's readout window did
            # not include the 48 rows, these variables will not be reset.
            # newyoffset is used to calculate l_ycenter - MS
            newyoffset = 0
            if (ishamamatsu) {
                det_roi_y_start = 0

                keypar (l_image//"[0]", "DETRO1Y", silent+)
                if (keypar.found) {
                    det_roi_y_start = int(keypar.value)
                } else {
                    print ("GDISPLAY: ERROR - DETRO1Y keyword not found")
                    goto crash
                }

                # This is for non-mosaiced images - if they are mosaiced
                # already ycenter gets reset later on - MS
                # This tests for trimmed images
                if ( (det_roi_y_start != y1 )) {
                    newyoffset = y1/2.0
                    Yoff = nint(imY - Ymaxtrim) / 2.
                # For images that were taken with a readout window that does
                # not overlap the lower 48 rows.
                } else if (det_roi_y_start > (row_removal_value / Ybin)){
                    newyoffset = (row_removal_value / Ybin)
                    Yoff = nint(imY - Ymaxtrim) / 2.
                }
            }

            # Added newyoffset variable due to GMOS-N new Hamamatsu CCDs - MS
            l_ycenter = ((y2-y1)/2. + y1 - newyoffset + Yoff) / Ymax

            # Reset xcenter,ycenter if image is binned
            l_xcenter = (l_xcenter-0.5)/Xbin + 0.5
            l_ycenter = (l_ycenter-0.5)/Ybin + 0.5

            # Reset magnification if higher resolution can be displayed
            # Never display more than 1pix to 1pix
            if ((min(Xbin,Ybin) > 1.) && (l_mag*min(Xbin,Ybin) <= 1.) && \
                l_fl_chmag) {

                l_mag = l_mag * min(Xbin,Ybin)
                l_fl_chmag = no  # don't check/change l_mag in the next rounds
                l_fl_mag = yes   # update xcenter,ycenter in all rounds
            }

            # Adjust xcenter,ycenter if l_mag has been changed
            if (l_fl_mag) {
                l_xcenter = (l_xcenter-0.5) * min(Xbin,Ybin) + 0.5
                l_ycenter = (l_ycenter-0.5) * min(Xbin,Ybin) + 0.5
            }

            if (l_fl_sat)
                dbpm = tmpxbpm//"["//str(n_i)//"]"//l_datasec
            if (l_signal != INDEF)
                doverlay = tmpxsig//"["//str(n_i)//"]"//l_datasec
            if (l_fl_sat && (n_k == 1) && !ise2vDD) {
                print ("GDISPLAY: Saturated pixels (>"//int(l_saturation)//" \
                    counts) flagged as red")
            }

            if ((l_signal != INDEF) && (n_k == 1))
                print ("GDISPLAY: Pixels with signal>="//str(l_signal)//\
                    " above bias flagged as green")
            printf ("%s %4d %4d %4d %4d  %5.1f  ", l_image//"["//str(n_i)//"]",
                x1, x2, y1, y2, l_mag)

            # If displaying a previously mosacied image (n_ext==1) reset
            # (x/y)center values to 0.5, as in the l_fl_paste is true (gmosaic)
            # case. Any change in magnification should not affect the centering
            # - MS
            if (n_ext == 1){
                l_xcenter = 0.5
                l_ycenter = 0.5
            }

            display (l_image//"["//str(n_i)//"]"//l_datasec, l_frame,
                erase=fl_erase, xcenter=l_xcenter, ycenter=l_ycenter, z1=l_z1,
                z2=l_z2, zrange=fl_zrange, zscale=fl_zscale, xmag=l_mag,
                ymag=l_mag, fill=no, bpm=dbpm, bpdisplay="overlay",
                bpcolors="red", overlay=doverlay, ocolors="green")

        } else if (l_fl_sec) {
            # Setup parameters for gmosaic, get EXTNAME,EXTVER in header
            if (n_k == 1) {
                copy (l_image, tmpim//".fits", verbose-)
                l_paste = tmpim
            }
            if (l_fl_bias) {
                imarith (l_paste//"["//str(n_i)//"]", "-", l_bias[n_k],
                    l_paste//"["//str(n_i)//",overwrite+]")
            }
            if (l_sci_ext == "") {
                gemhedit (l_paste//"["//str(n_i)//"]", "EXTNAME", "SCI", "",
                    delete-)
                gemhedit (l_paste//"["//str(n_i)//"]", "EXTVER", str(n_k), "",
                    delete-)
            }
        }

    } # end of while loop
    scanfile = ""
    flpr # make sure we can use EXTNAME,EXTVER for access

    if (l_sci_ext == "")
       l_sci_ext = "SCI"

    delete (tmpin, verify-, >& "dev$null")

    if (n_ext == 1 && l_fl_imexam) {
        print ("GDISPLAY: Starting imexamine - quit with q")

        if (l_z1 == 0. && l_z2 == 0.) {
            imexam (l_image//"["//str(n_i)//"]"//l_datasec, frame=l_frame)
        } else {
            imexam (l_image//"["//str(n_i)//"]"//l_datasec, frame=l_frame,
                display="display(image='$1',frame=$2,zr-,zs-,z1="//l_z1//\
                ",z2="//l_z2//")")
        }
    }

    if (l_fl_paste) {
        print ("GDISPLAY: Pasting images")
        gmosaic (l_paste, outimages=tmpout, fl_paste+, gap=gapvalue,
            fl_fixpix-, verbose=l_verbose, fl_clean-, sci_ext=l_sci_ext,
            fl_real-, logfile="dev$null")

        if (l_fl_sat) {
            tmpbpm = mktemp("tmpbpm")
            print ("GDISPLAY: Saturated pixels flagged as red")
            gmosaic (tmpxbpm, outimages=tmpbpm, fl_paste+, gap=gapvalue,
                fl_fixpix-, geointer="nearest", verbose=l_verbose, fl_clean-,
                sci_ext="DQ", fl_real-, logfile="dev$null")
            dbpm = tmpbpm//"[DQ,1]"
        } else
            dbpm = ""

        if (l_signal != INDEF) {
            tmpsignal = mktemp ("tmpsignal")
            print ("GDISPLAY: Pixels with signal>="//str(l_signal)//\
                " above bias flagged as green")
            gmosaic (tmpxsig, outimages=tmpsignal, fl_paste+, gap=gapvalue,
                fl_fixpix-, geointer="nearest", verbose=l_verbose, fl_clean-,
                sci_ext="VAR", fl_real-, logfile="dev$null")
            doverlay = tmpsignal//"[VAR,1]"
        } else
            doverlay = ""
        flpr # to get access through VAR,DQ after hedit

        display (tmpout//"[SCI,1]", l_frame, erase=yes, xcenter=0.5,
            ycenter=0.5,
            xmag=1, ymag=1, z1=l_z1, z2=l_z2, zrange=fl_zrange,
            zscale=fl_zscale, fill=no, bpm=dbpm, bpdisplay="overlay",
            bpcolors="red", overlay=doverlay, ocolors="green")

        if (l_fl_imexam) {
            print ("GDISPLAY: Starting imexamine - quit with q")
            imexamine (tmpout//"[SCI,1]", frame=l_frame)
        }
        if (l_output == "")
            imdelete (tmpout, verify-, >& "dev$null")
        else
            imrename (tmpout, l_output, verbose+)

    }

    goto clean

crash:
    # Exit with error subroutine
    status = 1
clean:
    imdelete (tmpim//","//tmpsignal//","//tmpbpm//","//tmpxbpm//","//tmpxsig,
        verify-, >& "dev$null")

end
